@@ -158,12 +158,8 @@ def test_default_format(env: LoggingEnvironment, class_: type[BaseJsonFormatter]
 
 @pytest.mark.parametrize("class_", ALL_FORMATTERS)
 def test_percentage_format(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
-    env.set_formatter(
-        class_(
-            # All kind of different styles to check the regex
-            "[%(levelname)8s] %(message)s %(filename)s:%(lineno)d %(asctime)"
-        )
-    )
+    # Note: We use different %s styles in the format to check the regex correctly collects them
+    env.set_formatter(class_("[%(levelname)8s] %(message)s %(filename)s:%(lineno)d %(asctime)"))
 
     msg = "testing logging format"
     env.logger.info(msg)
@@ -174,6 +170,58 @@ def test_percentage_format(env: LoggingEnvironment, class_: type[BaseJsonFormatt
     return
 
 
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_comma_format(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    # Note: we have double comma `,,` to test handling "empty" names
+    env.set_formatter(class_("levelname,,message,filename,lineno,asctime,", style=","))
+
+    msg = "testing logging format"
+    env.logger.info(msg)
+    log_json = env.load_json()
+
+    assert log_json["message"] == msg
+    assert log_json.keys() == {"levelname", "message", "filename", "lineno", "asctime"}
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_sequence_list_format(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(["levelname", "message", "filename", "lineno", "asctime"]))
+
+    msg = "testing logging format"
+    env.logger.info(msg)
+    log_json = env.load_json()
+
+    assert log_json["message"] == msg
+    assert log_json.keys() == {"levelname", "message", "filename", "lineno", "asctime"}
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_sequence_tuple_format(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(("levelname", "message", "filename", "lineno", "asctime")))
+
+    msg = "testing logging format"
+    env.logger.info(msg)
+    log_json = env.load_json()
+
+    assert log_json["message"] == msg
+    assert log_json.keys() == {"levelname", "message", "filename", "lineno", "asctime"}
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_defaults_field(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(defaults={"first": 1, "second": 2}))
+
+    env.logger.info("testing defaults field", extra={"first": 1234})
+    log_json = env.load_json()
+
+    assert log_json["first"] == 1234
+    assert log_json["second"] == 2
+    return
+
+
 @pytest.mark.parametrize("class_", ALL_FORMATTERS)
 def test_rename_base_field(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
     env.set_formatter(class_(rename_fields={"message": "@message"}))
@@ -186,6 +234,20 @@ def test_rename_base_field(env: LoggingEnvironment, class_: type[BaseJsonFormatt
     return
 
 
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_rename_with_defaults(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    """Make sure that the default fields are also renamed."""
+    env.set_formatter(class_(rename_fields={"custom": "@custom"}, defaults={"custom": 1234}))
+
+    msg = "testing rename with defaults"
+    env.logger.info(msg)
+    log_json = env.load_json()
+
+    assert log_json["@custom"] == 1234
+    assert "custom" not in log_json
+    return
+
+
 @pytest.mark.parametrize("class_", ALL_FORMATTERS)
 def test_rename_missing(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
     env.set_formatter(class_(rename_fields={"missing_field": "new_field"}))
@@ -321,6 +383,19 @@ def test_log_dict(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
     return
 
 
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_log_dict_defaults(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(defaults={"d1": 1234, "d2": "hello"}))
+
+    msg = {"d2": "world"}
+    env.logger.info(msg)
+    log_json = env.load_json()
+
+    assert log_json["d1"] == 1234
+    assert log_json["d2"] == "world"
+    return
+
+
 @pytest.mark.parametrize("class_", ALL_FORMATTERS)
 def test_log_extra(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
     env.set_formatter(class_())
@@ -341,9 +416,9 @@ def test_log_extra(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
 def test_custom_logic_adds_field(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
     class CustomJsonFormatter(class_):  # type: ignore[valid-type,misc]
 
-        def process_log_record(self, log_record):
-            log_record["custom"] = "value"
-            return super().process_log_record(log_record)
+        def process_log_record(self, log_data):
+            log_data["custom"] = "value"
+            return super().process_log_record(log_data)
 
     env.set_formatter(CustomJsonFormatter())
     env.logger.info("message")
@@ -529,8 +604,14 @@ def test_common_types_encoded(
     if pythonjsonlogger.MSGSPEC_AVAILABLE and class_ is MsgspecFormatter:
         # Dataclass: https://github.com/jcrist/msgspec/issues/681
         # Enum: https://github.com/jcrist/msgspec/issues/680
-        if obj is SomeDataclass or (
-            isinstance(obj, enum.Enum) and obj in {MultiEnum.BYTES, MultiEnum.NONE, MultiEnum.BOOL}
+        # These have been fixed in msgspec 0.19.0, however they also dropped python 3.8 support.
+        # https://github.com/jcrist/msgspec/releases/tag/0.19.0
+        if sys.version_info < (3, 9) and (
+            obj is SomeDataclass
+            or (
+                isinstance(obj, enum.Enum)
+                and obj in {MultiEnum.BYTES, MultiEnum.NONE, MultiEnum.BOOL}
+            )
         ):
             pytest.xfail()
 
@@ -577,6 +658,55 @@ def test_custom_default(env: LoggingEnvironment, class_: type[BaseJsonFormatter]
     return
 
 
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_exc_info_as_array(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(exc_info_as_array=True))
+
+    try:
+        raise Exception("Error")
+    except BaseException:
+        env.logger.exception("Error occurs")
+    log_json = env.load_json()
+
+    assert isinstance(log_json["exc_info"], list)
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_exc_info_as_array_no_exc_info(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(exc_info_as_array=True))
+
+    env.logger.info("hello")
+    log_json = env.load_json()
+
+    assert "exc_info" not in log_json
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_stack_info_as_array(env: LoggingEnvironment, class_: type[BaseJsonFormatter]):
+    env.set_formatter(class_(stack_info_as_array=True))
+
+    env.logger.info("hello", stack_info=True)
+    log_json = env.load_json()
+
+    assert isinstance(log_json["stack_info"], list)
+    return
+
+
+@pytest.mark.parametrize("class_", ALL_FORMATTERS)
+def test_stack_info_as_array_no_stack_info(
+    env: LoggingEnvironment, class_: type[BaseJsonFormatter]
+):
+    env.set_formatter(class_(stack_info_as_array=True))
+
+    env.logger.info("hello", stack_info=False)
+    log_json = env.load_json()
+
+    assert "stack_info" not in log_json
+    return
+
+
 ## JsonFormatter Specific
 ## -----------------------------------------------------------------------------
 def test_json_ensure_ascii_true(env: LoggingEnvironment):
